//#############################################################################
//#############################################################################
//    File:    graph.cpp
//    Authors: Yan Xu, Ted Ralphs, Matt Galati
//    Date:    03/13/03(First verstion)
//    Course:  IE 170, Lab 7
//    Purpose: This is the definition file for a graph
//#############################################################################
//#############################################################################

#include <iostream>

#include "graph.h"

//#############################################################################
//#############################################################################

Graph::Graph(std::istream& is)
{
  //===========================================================================
  // Set up the vertex list. To simplify matters, we will assume that the
  // vertices are indexed 0 to vertNum-1.
  //===========================================================================

  // Fill this part in.

  //===========================================================================
  // Read in the list of edges and set up the adjacency lists. Remember that
  // you have to put each edge on two different adjacency lists. For
  // simplicity, we will assume that EndPoint1 is the vertex on whose
  // adjacency list the edge appears, so the two edges are not identical!
  //===========================================================================

  // Fill this part in.
}

//#############################################################################
//#############################################################################

Graph::~Graph()
{
   if (vertNum_ != 0 && vertices_ != 0){
      for (int i = 0; i < vertNum_; i++){
	 delete vertices_[i];
      }
      delete [] vertices_;
   }
}

//#############################################################################
//#############################################################################

void Graph::print()
{ 
  std::cout << "HEAD" << "\t                  TAIL     " << std::endl;
  std::cout << "----" << "\t----------------------------------------------"
	    << std::endl;

  node<Edge> * headNode;
  Edge * ed;

  for (int i = 0; i < vertNum_; ++i) {
    std::cout << vertices_[i]->getIndex() << "\t";
    headNode =  vertices_[i]->getAdjList().getHead();
 
    while(headNode != 0) {
      ed = &headNode->getData();
      int head = ed->getEndPoint1();
      int tail = ed->getEndPoint2();
      
      std::cout << tail << "\t";
      headNode = headNode->getNext();
    }

    std::cout << std::endl;
  }  

}

//#############################################################################
//#############################################################################

void Graph::clear()
{
  int i; 
  // clear vertex data
  for (i = 0; i < vertNum_; ++i) {
    vertices_[i]->setColor(WHITE);
    vertices_[i]->setPred(0);
    vertices_[i]->setDistance(INT_MAX);
    vertices_[i]->setDiscovered(INT_MAX);
    vertices_[i]->setFinished(INT_MAX);
  }
}

//#############################################################################
//  This function implements a general graph search. Note that we can replace
//  the stack below with any kind of a priority queue and the algorithm will
//  work without modification. In the next lab, we will be doing just that.
//#############################################################################

void Graph::search()
{

  
  clear();    // Make sure graph is unmarked and proper initailized

  std::cout << "\nGraph search: " << std::endl << std::endl;
  std::cout << "VERTEX" << "\tDISTANCE" << std::endl;
  std::cout << "------" << "\t--------" << std::endl;

  for (int i = 0; i < vertNum_; ++i) {
    // Look for the starting vertex
    if ( vertices_[i]->getColor() == WHITE ){
      // OK, found the starting vertex. Now put it on the stack.
      Vertex* source = vertices_[i];
      source->setColor(GRAY);
      source->setDistance(0);
      stack< Vertex* > vertList;
      vertList.push(source);
      while (vertList.empty() != true) {
	Edge * ed;
	Vertex* v;
	// Get first vertex in vertList
	Vertex* u = vertList.top();       
	// Remove first vertex in vertList
	vertList.pop();                     
	// Now put all the unprocessed neighbors on the stack.
	node<Edge> * head;
	for (head = u->getAdjList().getHead(); head != 0; 
	     head = head->getNext()){
	   // First get the adjacent vertex (assuming vertices are indexed 0
	   // through vertNum_-1).
	   ed = &(head->getData());
	   v = vertices_[ed->getEndPoint2()];
	   // If it's white, put it on the list
	   if (v->getColor() == WHITE) {
	      v->setColor(GRAY);
	      v->setDistance(u->getDistance() + 1);
	      v->setPred(u);
	      vertList.push(v);
	   }
	}
	// Done with this vertex
	u->setColor(BLACK);
	std::cout << u->getIndex() <<"\t" << u->getDistance() << std::endl;
      }
    }
  }
}

//#############################################################################

