//#############################################################################
//#############################################################################
//    File:    graph.h
//    Author:  Yan Xu, Ted Ralphs, Matt Galati
//    Date:    03/13/03(First verstion)
//    Course:  IE 170, Lab 7
//    Purpose: This is the declaration file for a graph
//#############################################################################
//#############################################################################

#ifndef graph_h
#define graph_h

#include <iostream>
#include "list.hpp"
#include "stack.hpp"


//#############################################################################

enum COLOR { BLACK, GRAY, WHITE };

//#############################################################################
// This class declares a edge in the graph/adjacent lists
//#############################################################################

class Edge {
 private:
  int     endPoint1_; // The vertex on whose list this edge resides
  int     endPoint2_; // The other vertex

 public:
  Edge() : endPoint1_(0), endPoint2_(0) {}
  Edge(int e1, int e2) : endPoint1_(e1), endPoint2_(e2) {}
  ~ Edge() {}

  //---------------------------------------------------------------------------
  // Modifying functions
  void    setEndPoint1(int e)     { endPoint1_ = e; }
  void    setEndPoint2(int e)     { endPoint2_ = e; }

  //---------------------------------------------------------------------------
  // Query functions
  int     getEndPoint1()          { return endPoint1_; }
  int     getEndPoint2()          { return endPoint2_; }
};


//#############################################################################
// This class declares a vertex in a graph
//#############################################################################

class Vertex {
 private:
  
  int          index_;          // The node ID in the graph
  Vertex*      pred_;           // Predecessor node
  COLOR        color_;          // Color the vertex
  list <Edge>  adjList_;        // Adjacent list for this vertex

  // Fields needed for specific applications
  int          distance_;       // distance from source node
  int          discovered_;     // the time discovered in the search
  int          finished_;       // the time finish examining in the search

 public:
  Vertex() : index_(0), distance_(INT_MAX), pred_(0),
    color_(WHITE), discovered_(INT_MAX), finished_(INT_MAX) {}
  
  Vertex(int i) : index_(i), pred_(0),
    color_(WHITE), distance_(0),
    discovered_(INT_MAX), finished_(INT_MAX) {}

  ~Vertex() {}

  //---------------------------------------------------------------------------
  // Modification functions
  //---------------------------------------------------------------------------

  void         setIndex(int e)          { index_ = e; }
  void         setPred(Vertex* p)       { pred_ = p; }
  void         setColor(COLOR c)        { color_ =  c; }

  void         setDistance(int d)       { distance_ = d; }
  void         setDiscovered(int t)     { discovered_ = t; }
  void         setFinished(int t)       { finished_ = t; }

  //---------------------------------------------------------------------------
  // Query functions
  //---------------------------------------------------------------------------

  int          getIndex()              { return index_; }
  Vertex*      getPred()               { return pred_; }
  COLOR        getColor()              { return color_; }
  list <Edge>& getAdjList()            { return adjList_; }
  
  int          getDistance()           { return distance_; }
  int          getDiscovered()         { return discovered_; }
  int          getFinished()           { return finished_; }

};


//#############################################################################
// A graph and related algorithms.
//#############################################################################

class Graph {
 private:
  int       vertNum_;    // the number of vertices in the graph
  int       edgeNum_;    // the number of edges in the graph

  Vertex**  vertices_;   // All the vertice in the graph, every vertex has 
                         // an adjacent list

 public:
  Graph() : vertNum_(0), edgeNum_(0), vertices_(0) {}
  Graph(std::istream& is);
  ~Graph();

  //---------------------------------------------------------------------------
  // Modifying functions
  void      setVertNum(int n)         { vertNum_ = n; }
  void      setEdgeNum(int n)         { edgeNum_ = n; }

  //---------------------------------------------------------------------------
  // Query functions
  int       getVertNum()              { return vertNum_; }
  int       getEdgeNum()              { return edgeNum_; }

  //---------------------------------------------------------------------------
  // Graph Algorithms 
  void      print();
  void      clear();       // Clear exiting graph data
  void      search();

};

//#############################################################################
#endif
